import java.awt.Button;

public class Endpoint
{
	// the states an endpoint can be in
	public final static int AVAILABLE = 0;
	public final static int TALKING = 1;

	public String epid;
	public String alias;
	public Button button;

	public Endpoint(String _epid, String _alias, Button _button)
	{
		epid = _epid;
		alias = _alias;
		button = _button;
	}
}
